@@ -61,6 +61,9 @@ unsigned long * volatile fut_next_slot = NULL;
 /*	points to byte beyond end of buffer (multiple of 4) */
 unsigned long * volatile fut_last_slot = NULL;
 
+/*	Protects the fut_*_slot pointers */
+pthread_spinlock_t fut_slot_lock;
+
 fxt_t fut;
 static struct fxt_infos *fut_infos;
 
@@ -73,7 +76,7 @@ static unsigned int	nallocated = 0;
 /* filename of the output trace */
 static char *fut_filename = NULL;
 /* file descriptor of the output trace */
-static int fut_fd = 0;
+static int fut_fd = -1;
 
 /* if !=0, activate the recording of thread id */
 int record_tid_activated = 0;
@@ -200,6 +203,7 @@ int fut_setup( unsigned int nlongs, unsigned int keymask, unsigned int threadid
 	dumptime(&fut_infos->start_time, &fut_infos->start_jiffies);
 
 	fut_next_slot = (unsigned long *)bufptr;
+	pthread_spin_init(&fut_slot_lock, 0);
 	fut_active = keymask & FULL_ACTIVE_MASK;
 
 	FUT_PROBE1(FUT_GCC_INSTRUMENT_KEYMASK,
@@ -362,7 +366,7 @@ int fut_flush( char* filename, void* next_slot, int record_flush_events )
 
   size = nlongs * sizeof(long);
 
-  if(!fut_fd) {
+  if(fut_fd == -1) {
 
     /* First time fut_flush is called. Open the output file and write headers */
     if( fut_filename == NULL )
@@ -399,7 +403,7 @@ int fut_flush( char* filename, void* next_slot, int record_flush_events )
 
   fxt_trace_user_raw_t start_flush_event;
   if(record_flush_events)
-    __fut_record_event(&start_flush_event, ((FUT_START_FLUSH_CODE)<<8) | 0x01);
+    __fut_record_event(&start_flush_event, fut_getstamp(), ((FUT_START_FLUSH_CODE)<<8) | 0x01);
 
   /* dump the buffer to disk */
   if( write(fut_fd, (void *)copy, size) < 0 ) {
@@ -408,7 +412,7 @@ int fut_flush( char* filename, void* next_slot, int record_flush_events )
 
   fxt_trace_user_raw_t stop_flush_event;
   if(record_flush_events) {
-    __fut_record_event(&stop_flush_event, ((FUT_STOP_FLUSH_CODE)<<8) | 0x01);
+    __fut_record_event(&stop_flush_event, fut_getstamp(), ((FUT_STOP_FLUSH_CODE)<<8) | 0x01);
 
     if( write(fut_fd, (void *)&start_flush_event, FUT_SIZE(0)) < 0 ) {
 	perror("write buffer");
@@ -457,6 +461,8 @@ int fut_endup( char *filename )
 	  if( close(fut_fd) < 0 )
 	    perror(fut_filename);
 
+	  fut_fd = -1;
+
 	  __fut_reset_pointers();
 	  //fut_done(); // Removed by Raymond
 	  fut_active = 1;
